@@ -1,6 +1,7 @@
 /****************************************************************************
  *    Copyright 2011  Ian Wadham <iandw.au@gmail.com>                       *
  *    Copyright 2006  David Bau <david bau @ gmail com> Original algorithms *
+ *    Copyright 2015  Ian Wadham <iandw.au@gmail.com>                       *
  *                                                                          *
  *    This program is free software; you can redistribute it and/or         *
  *    modify it under the terms of the GNU General Public License as        *
@@ -17,19 +18,22 @@
  ****************************************************************************/
 
 #include "debug.h"
+#include "ksudoku_logging.h"
 
 #include "sudokuboard.h"
 #include "state.h"
+#include "mathdokugenerator.h"
 
-#include <KLocale>
+#include <KLocalizedString>
 #include <KMessageBox>
+#include <KStandardGuiItem>
 
+#include <QDebug>
 #include <QMultiMap>
-
 #include <QTime>
 
-#include <stdio.h>
-#include <time.h>
+#include <cstdio>
+#include <ctime>
 
 SudokuBoard::SudokuBoard (SKGraph * graph)
     :
@@ -45,42 +49,81 @@ SudokuBoard::SudokuBoard (SKGraph * graph)
     m_vacant       (VACANT),
     m_unusable     (UNUSABLE)
 {
-    dbgLevel = 0;
-
     m_stats.type      = m_type;
     m_stats.blockSize = m_blockSize;
     m_stats.order     = m_order;
     m_boardSize       = graph->sizeX(); // TODO - IDW. Rationalise grid sizes.
-    dbe "SudokuBoard: type %d %s, block %d, order %d, BoardArea %d\n",
-	m_type, graph->name().toAscii().constData(),
-        m_blockSize, m_order, m_boardArea);
+    qCDebug(KSudokuLog) << "SudokuBoard: type " << m_type << graph->name()
+                        << ", block " << m_blockSize << ", order " << m_order
+                        << ", BoardArea " << m_boardArea;
 }
 
 void SudokuBoard::setSeed()
 {
     static bool started = false;
     if (started) {
-        dbo1 "setSeed(): RESET IS TURNED OFF\n");
+        qCDebug(KSudokuLog) << "setSeed(): RESET IS TURNED OFF";
         // qsrand (m_stats.seed); // IDW test.
     }
     else {
         started = true;
-        m_stats.seed = time(0);
+        m_stats.seed = std::time(nullptr);
         qsrand (m_stats.seed);
-        dbo1 "setSeed(): SEED = %d\n", m_stats.seed);
+        qCDebug(KSudokuLog) << "setSeed(): SEED = " << m_stats.seed;
     }
 }
 
-void SudokuBoard::generatePuzzle (BoardContents & puzzle,
-                                  BoardContents & solution,
-                                  Difficulty difficultyRequired,
-                                  Symmetry symmetry)
+bool SudokuBoard::generatePuzzle             (BoardContents & puzzle,
+                                              BoardContents & solution,
+                                              Difficulty difficultyRequired,
+                                              Symmetry symmetry)
 {
-    dbe "Entered generatePuzzle(): difficulty %d, symmetry %d\n",
-        difficultyRequired, symmetry);
-    QTime t;
-    t.start();
+    qCDebug(KSudokuLog) << "Entered generatePuzzle(): difficulty "
+                        << difficultyRequired << ", symmetry " << symmetry;
     setSeed();
+
+    SudokuType puzzleType = m_graph->specificType();
+    if ((puzzleType == Mathdoku) || (puzzleType == KillerSudoku)) {
+	// Generate variants of Mathdoku (aka KenKen TM) or Killer Sudoku types.
+	int maxTries = 10;
+	int numTries = 0;
+	bool success = false;
+	while (true) {
+	    MathdokuGenerator mg (m_graph);
+	    // Find numbers to satisfy Sudoku rules: they will be the solution.
+	    solution = fillBoard();
+	    // Generate a Mathdoku or Killer Sudoku puzzle having this solution.
+	    numTries++;
+	    success = mg.generateMathdokuTypes (puzzle, solution,
+				    &m_KSudokuMoves, difficultyRequired);
+	    if (success) {
+		return true;
+	    }
+	    else if (numTries >= maxTries) {
+		QWidget owner;
+		if (KMessageBox::questionYesNo (&owner,
+			    i18n("Attempts to generate a puzzle failed after "
+				 "about 200 tries. Try again?"),
+			    i18n("Mathdoku or Killer Sudoku Puzzle"))
+			    == KMessageBox::No) {
+		    return false;	// Go back to the Welcome screen.
+		}
+		numTries = 0;		// Try again.
+	    }
+	}
+    }
+    else {
+	// Generate variants of Sudoku (2D) and Roxdoku (3D) types.
+	return generateSudokuRoxdokuTypes (puzzle, solution,
+                                    difficultyRequired, symmetry);
+    }
+}
+
+bool SudokuBoard::generateSudokuRoxdokuTypes (BoardContents & puzzle,
+                                              BoardContents & solution,
+                                              Difficulty difficultyRequired,
+                                              Symmetry symmetry)
+{
     const int     maxTries = 20;
     int           count = 0;
     float         bestRating = 0.0;
@@ -91,47 +134,48 @@ void SudokuBoard::generatePuzzle (BoardContents & puzzle,
     BoardContents currPuzzle;
     BoardContents currSolution;
 
+    QTime t;
+    t.start();
     if (m_graph->sizeZ() > 1) {
 	symmetry = NONE;		// Symmetry not implemented in 3-D.
     }
     if (symmetry == RANDOM_SYM) {	// Choose a symmetry at random.
         symmetry = (Symmetry) (qrand() % (int) LAST_CHOICE);
     }
-    dbo1 "SYMMETRY IS %d\n", (int) symmetry);
+    qCDebug(KSudokuLog) << "SYMMETRY IS" << symmetry;
     if (symmetry == DIAGONAL_1) {
 	// If diagonal symmetry, choose between NW->SE and NE->SW diagonals.
         symmetry = (qrand() % 2 == 0) ? DIAGONAL_1 : DIAGONAL_2;
-	dbo1 "Diagonal symmetry, choosing %s\n",
-            (symmetry == DIAGONAL_1) ? "DIAGONAL_1" : "DIAGONAL_2");
+        qCDebug(KSudokuLog) << "Diagonal symmetry, choosing " <<
+            ((symmetry == DIAGONAL_1) ? "DIAGONAL_1" : "DIAGONAL_2");
     }
 
     while (true) {
         // Fill the board with values that satisfy the Sudoku rules but are
         // chosen in a random way: these values are the solution of the puzzle.
         currSolution = this->fillBoard();
-        dbo1 "RETURN FROM fillBoard()\n");
-        dbo1 "Time to fill board: %d msec\n", t.elapsed());
+        qCDebug(KSudokuLog) << "Return from fillBoard() - time to fill board:"
+                            << t.elapsed() << " msec";
 
         // Randomly insert solution-values into an empty board until a point is
         // reached where all the cells in the solution can be logically deduced.
         currPuzzle = insertValues (currSolution, difficultyRequired, symmetry);
-        dbo1 "RETURN FROM insertValues()\n");
-        dbo1 "Time to do insertValues: %d msec\n", t.elapsed());
+        qCDebug(KSudokuLog) << "Return from insertValues() - duration:"
+                            << t.elapsed() << " msec";
 
         if (difficultyRequired > m_stats.difficulty) {
             // Make the puzzle harder by removing values at random.
             currPuzzle = removeValues (currSolution, currPuzzle,
                                        difficultyRequired, symmetry);
-            dbo1 "RETURN FROM removeValues()\n");
-            dbo1 "Time to do removeValues: %d msec\n", t.elapsed());
+            qCDebug(KSudokuLog) << "Return from removeValues() - duration:"
+                                << t.elapsed() << " msec";
         }
 
         Difficulty d = calculateRating (currPuzzle, 5);
         count++;
-        dbo1 "CYCLE %d, achieved difficulty %d, required %d, rating %3.1f\n",
-                         count, d, difficultyRequired, m_accum.rating);
-        dbe1 "CYCLE %d, achieved difficulty %d, required %d, rating %3.1f\n",
-                         count, d, difficultyRequired, m_accum.rating);
+        qCInfo(KSudokuLog) <<  "CYCLE" << count << ", achieved difficulty" << d
+                           << ", required" << difficultyRequired << ", rating"
+                           << m_accum.rating;
 
 	// Use the highest rated puzzle so far.
 	if (m_accum.rating > bestRating) {
@@ -144,6 +188,9 @@ void SudokuBoard::generatePuzzle (BoardContents & puzzle,
 	    puzzle           = currPuzzle;
 	}
 
+	// Express the rating to 1 decimal place in whatever locale we have.
+	QString ratingStr = ki18n("%1").subs(bestRating, 0, 'f', 1).toString();
+	// Check and explain the Sudoku/Roxdoku puzzle-generator's results.
 	if ((d < difficultyRequired) && (count >= maxTries)) {
             // Exit after max attempts?
             QWidget owner;
@@ -155,9 +202,9 @@ void SudokuBoard::generatePuzzle (BoardContents & puzzle,
 			   "\n"
 			   "If you accept the puzzle, it may help to change to "
 			   "No Symmetry or some low symmetry type, then use "
-			   "Game->New and try generating another puzzle.")
-		      .arg(maxTries).arg(bestDifficulty)
-		      .arg(bestRating, 0, 'f', 1).arg(difficultyRequired),
+			   "Game->New and try generating another puzzle.",
+			   maxTries, bestDifficulty,
+			   ratingStr, difficultyRequired),
                       i18n("Difficulty Level"),
                       KGuiItem(i18n("&Try Again")), KGuiItem(i18n("&Accept")));
             if (ans == KMessageBox::Yes) {
@@ -170,19 +217,20 @@ void SudokuBoard::generatePuzzle (BoardContents & puzzle,
             QWidget owner;
 	    int ans = 0;
 	    if (m_accum.nGuesses == 0) {
-                const QString rating = QString("%1").arg(bestRating, 0, 'f', 1);
                 ans = KMessageBox::questionYesNo (&owner,
 		       i18n("It will be possible to solve the generated puzzle "
 			    "by logic alone. No guessing will be required.\n"
 			    "\n"
 			    "The internal difficulty rating is %1. There are "
-			    "%2 clues at the start and %3 moves to go.", rating, bestNClues, (m_stats.nCells - bestNClues)),
+			    "%2 clues at the start and %3 moves to go.",
+			    ratingStr, bestNClues,
+			    (m_stats.nCells - bestNClues)),
 		       i18n("Difficulty Level"),
-                       KGuiItem(i18n("&OK")), KGuiItem(i18n("&Retry")));
+                       KStandardGuiItem::ok(), KGuiItem(i18n("&Retry")));
 	    }
 	    else {
-                const QString average = QString::fromLatin1("%1").arg(((float) bestNGuesses) / 5.0, 0, 'f', 1);
-                const QString rating = QString::fromLatin1("%1").arg(bestRating, 0, 'f', 1);
+                QString avGuessStr = ki18n("%1").subs(((float) bestNGuesses) /
+			5.0, 0, 'f', 1).toString(); // Format as for ratingStr.
                 ans = KMessageBox::questionYesNo (&owner,
 		       i18n("Solving the generated puzzle will require an "
 			    "average of %1 guesses or branch points and if you "
@@ -190,9 +238,11 @@ void SudokuBoard::generatePuzzle (BoardContents & puzzle,
 			    "first guess should come after %2 moves.\n"
 			    "\n"
 			    "The internal difficulty rating is %3, there are "
-			    "%4 clues at the start and %5 moves to go.", average, bestFirstGuessAt, rating, (m_stats.nCells - bestNClues)),
+			    "%4 clues at the start and %5 moves to go.",
+			    avGuessStr, bestFirstGuessAt, ratingStr,
+			    bestNClues, (m_stats.nCells - bestNClues)),
                        i18n("Difficulty Level"),
-                       KGuiItem(i18n("&OK")), KGuiItem(i18n("&Retry")));
+                       KStandardGuiItem::ok(), KGuiItem(i18n("&Retry")));
 	    }
 	    // Exit when the required difficulty or number of tries is reached.
             if (ans == KMessageBox::No) {
@@ -208,12 +258,9 @@ void SudokuBoard::generatePuzzle (BoardContents & puzzle,
         }
     }
 
-    if (dbgLevel > 0) {
-        dbo "FINAL PUZZLE\n");
-        print (puzzle);
-        dbo "SOLUTION\n");
-        print (solution);
-    }
+    qCDebug(KSudokuLog) << "FINAL PUZZLE" << puzzle;
+
+    return true;
 }
 
 Difficulty SudokuBoard::calculateRating (const BoardContents & puzzle,
@@ -298,10 +345,7 @@ void SudokuBoard::getMoveList (QList<int> & moveList)
 BoardContents & SudokuBoard::solveBoard (const BoardContents & boardValues,
                                                GuessingMode gMode)
 {
-    if (dbgLevel >= 2) {
-        dbo "solveBoard()\n");
-        print (boardValues);
-    }
+    qCInfo(KSudokuLog) << "solveBoard()" << boardValues;
     m_currentValues = boardValues;
     return solve (gMode);
 }
@@ -418,13 +462,10 @@ BoardContents SudokuBoard::insertValues (const BoardContents & solution,
             changeClues (filled, cell, symmetry, solution);
 
             deduceValues (filled, Random /* NotRandom */);
-            if (dbgLevel >= 3) {
-                print (puzzle);
-                print (filled);
-            }
+            qCDebug(KSudokuLog) << "Puzzle:" << puzzle << "; filled" << filled;
         }
     }
-    if (dbgLevel > 0) print (puzzle);
+    qCDebug(KSudokuLog) << "Puzzle:" << puzzle;
 
     while (true) {
         // Check the difficulty of the puzzle.
@@ -447,7 +488,7 @@ BoardContents SudokuBoard::insertValues (const BoardContents & solution,
                 index, solution.at (cell),
                 cell, cell/m_boardSize + 1, cell%m_boardSize + 1);
     }
-    if (dbgLevel > 0) print (puzzle);
+    qCDebug(KSudokuLog) << "Puzzle:" << puzzle;
     return puzzle;
 }
 
@@ -521,10 +562,9 @@ BoardContents SudokuBoard::removeValues (const BoardContents & solution,
 
             else if (m_stats.difficulty > required) {
                 // Finish if the required difficulty is exceeded.
-                dbo1 "BREAK on difficulty %d\n", m_stats.difficulty);
-                dbe1 "BREAK on difficulty %d\n", m_stats.difficulty);
-                dbo1 "Replaced %d at cell %d, overshoot is %d\n",
-                        value, cell, tailOfRemoved.count());
+                qCInfo(KSudokuLog) << "Break on difficulty - replaced" << value
+                                   << "at cell" << cell << ", overshoot is"
+                                   << tailOfRemoved.count();
                 // Replace the value involved.
 	        changeClues (puzzle, cell, symmetry, solution);
                 break;
@@ -821,7 +861,7 @@ GuessesList SudokuBoard::deduceValues (BoardContents & boardValues,
 
                 guesses.clear();
                 for (int i = 0; i < original.count(); i++) {
-                    guesses.append (original.at (sequence.at (i))); 
+                    guesses.append (original.at (sequence.at (i)));
                 }
             }
             dbo2 "Guess    ");
@@ -856,7 +896,7 @@ void SudokuBoard::clear (BoardContents & boardValues)
 BoardContents & SudokuBoard::fillBoard()
 {
     // Solve the empty board, thus filling it with values at random.  These
-    // values can be the starting point for generating a puzzle and also the 
+    // values can be the starting point for generating a puzzle and also the
     // final solution of that puzzle.
 
     clear (m_currentValues);
@@ -960,7 +1000,7 @@ void SudokuBoard::setUpValueRequirements (BoardContents & boardValues)
 	    int cell = cellList.at (n);
             m_validCellValues [cell] &= m_requiredGroupValues.at (group);
             index++;
-        }   
+        }
     }
     dbo2 "Finished setUpValueRequirements()\n");
 
@@ -987,15 +1027,15 @@ void SudokuBoard::updateValueRequirements (BoardContents & boardValues, int cell
     m_validCellValues [cell] = 0;
 
     // Update the requirements for each group to which this cell belongs.
-    QList<int> groupList = m_graph->cliqueList(cell);
-    foreach (int group, groupList) {
+    const QList<int> groupList = m_graph->cliqueList(cell);
+    for (int group : groupList) {
         m_requiredGroupValues [group] &= bitPattern;
 
 	QVector<int> cellList = m_graph->clique (group);
         for (int n = 0; n < m_order; n++) {
 	    int cell = cellList.at (n);
             m_validCellValues [cell] &= bitPattern;
-        }   
+        }
     }
 }
 
@@ -1081,4 +1121,4 @@ int SudokuBoard::getSymmetricIndices
     return result;
 }
 
-#include "sudokuboard.moc"
+
