@@ -20,6 +20,7 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
+#include "ksudoku_logging.h"
 #include "ksudokugame.h"
 
 #include "puzzle.h"
@@ -29,13 +30,11 @@
 #include "globals.h"
 
 #include <KMessageBox>
-#include <KLocale>
+#include <KLocalizedString>
 
+#include <QDebug>
 #include <QList>
-
-#include <QDebug> // IDW
-
-#include <kurl.h>
+#include <QUrl>
 
 class QWidget;
 
@@ -57,10 +56,10 @@ private:
 	int m_refCount;
 	
 public: // The slots of GameIFace
-	void undo();
-	void redo();
-	void addCheckpoint();
-	void undo2Checkpoint();
+	void undo() override;
+	void redo() override;
+	void addCheckpoint() override;
+	void undo2Checkpoint() override;
 	
 public:
 	inline void emitModified(bool isModified) { emit modified(isModified); }
@@ -81,7 +80,7 @@ public:
 	Puzzle* puzzle;
 	QTime time;
 	int   accumTime;
-	KUrl url;
+	QUrl url;
 	QList<HistoryEvent> history;
 	int historyPos;
 
@@ -193,7 +192,7 @@ bool Game::simpleCheck() const { // IDW TODO - This does nothing useful now
                                  //            that connections have gone.
 	if(!m_private) return false;
 
-	qDebug() << "BYPASSED Game::simpleCheck()";
+	qCDebug(KSudokuLog) << "BYPASSED Game::simpleCheck()";
 	return true; // IDW: disabled rest of test.
 
 // IDW test. Eliminated optimized[] arrays and xxxConnection() functions.
@@ -218,14 +217,14 @@ Puzzle* Game::puzzle() const {
 	return m_private->puzzle;
 }
 
-void Game::setUrl(const KUrl& url) {
+void Game::setUrl(const QUrl& url) {
 	if(!m_private) return;
 	
 	m_private->url = url;
 }
 
-KUrl Game::getUrl() const {
-	if(!m_private) return KUrl();
+QUrl Game::getUrl() const {
+	if(!m_private) return QUrl();
 	return m_private->url;
 }
 
@@ -307,7 +306,7 @@ bool Game::addToCage (int pos, int val)
 	    return false;	// We are not keying in a cage.
 	}
 #ifdef MATHDOKUENTRY_LOG
-	qDebug() << "\nGame::addToCage: pos" << pos << "action" << val;
+	qCDebug(KSudokuLog) << "Game::addToCage: pos" << pos << "action" << val;
 #endif
 	if (! m_private->m_currentCageSaved) {	// Start a new cage.
 	    m_private->m_cage.clear();
@@ -379,7 +378,7 @@ bool Game::addToCage (int pos, int val)
 	if (m_private->m_currentCageSaved) {	// If new cage, skip dropping.
 	    int cageNum = g->cageCount() - 1;
 #ifdef MATHDOKUENTRY_LOG
-	    qDebug() << "    DROPPING CAGE" << cageNum
+	    qCDebug(KSudokuLog) << "    DROPPING CAGE" << cageNum
 		     << "m_currentCageSaved" << m_private->m_currentCageSaved
 		     << "m_cage" << m_private->m_cage;
 #endif
@@ -389,7 +388,7 @@ bool Game::addToCage (int pos, int val)
 	g->addCage (m_private->m_cage,
 		    m_private->m_cageOperator, m_private->m_cageValue);
 #ifdef MATHDOKUENTRY_LOG
-	qDebug() << "    ADDED CAGE" << (g->cageCount() - 1)
+	qCDebug(KSudokuLog) << "    ADDED CAGE" << (g->cageCount() - 1)
 		 << "value" << m_private->m_cageValue
 		 << "op" << m_private->m_cageOperator
 		 << m_private->m_cage;
@@ -454,9 +453,9 @@ bool Game::validCell (int pos, SKGraph * g)
 void Game::finishCurrentCage (SKGraph * g)
 {
 #ifdef MATHDOKUENTRY_LOG
-	qDebug() << "END CAGE: value" << m_private->m_cageValue
+	qCDebug(KSudokuLog) << "END CAGE: value" << m_private->m_cageValue
 		 << "op" << m_private->m_cageOperator
-		 << m_private->m_cage << "\n";
+		 << m_private->m_cage;
 #endif
 	// If Killer Sudoku and cage-size > 1, force operator to be +.
 	if ((g->specificType() == KillerSudoku) &&
@@ -527,7 +526,7 @@ void Game::deleteCageAt (int pos, SKGraph * g)
 	    if (cageNumP1 > 0) {
 		if(KMessageBox::questionYesNo (messageParent(),
 		       i18n("Do you wish to delete this cage?"),
-		       i18n("Delete Cage"), KGuiItem(i18n("Delete")),
+		       i18n("Delete Cage"), KStandardGuiItem::del(),
 		       KStandardGuiItem::cancel(), QString("CageDelConfirm"))
 		       == KMessageBox::No) {
 		    return;
@@ -542,7 +541,7 @@ void Game::deleteCageAt (int pos, SKGraph * g)
 		m_private->emitCageChange (-cageNumP1, false);
 		// Remove the cage from the puzzle's graph.
 #ifdef MATHDOKUENTRY_LOG
-		qDebug() << "    DROP CAGE" << (cageNumP1 - 1);
+		qCDebug(KSudokuLog) << "    DROP CAGE" << (cageNumP1 - 1);
 #endif
 		g->dropCage (cageNumP1 - 1);
 		if (m_private->m_cage.indexOf (pos) >= 0) {
@@ -562,7 +561,7 @@ void Game::deleteCageAt (int pos, SKGraph * g)
 		i18n("The Delete action finds that there are no cages "
 		     "to delete."), i18n("Delete Cage"));
 #ifdef MATHDOKUENTRY_LOG
-	    qDebug() << "NO CAGES TO DELETE.";
+	    qCDebug(KSudokuLog) << "NO CAGES TO DELETE.";
 #endif
 	}
 }
@@ -697,7 +696,7 @@ const BoardContents Game::allValues() const {
 
 QTime Game::time() const {
 	if(!m_private) return QTime();
-	return QTime().addMSecs(msecsElapsed());
+	return QTime(0,0).addMSecs(msecsElapsed());
 }
 
 int Game::msecsElapsed() const {
@@ -780,4 +779,4 @@ QWidget * Game::messageParent()
 
 }
 
-#include "ksudokugame.moc"
+
