@@ -21,54 +21,67 @@
  ***************************************************************************/
 
 #include "ksudoku.h"
-#include <kapplication.h>
-//#include <dcopclient.h>
-#include <kaboutdata.h>
-#include <kcmdlineargs.h>
-#include <klocale.h>
-#include <kconfigdialogmanager.h>
+
+#include <KAboutData>
+#include <KCrash>
+#include <KLocalizedString>
+#include <KConfigDialogManager>
+
+#include <QApplication>
+#include <QCommandLineParser>
+#include <QCommandLineOption>
+#include <QDir>
+#include <QUrl>
 
 #include <cstdlib>
-#include <time.h>
-
-#include <KUrl>
+#include <ctime>
 
 
 static const char description[] =
     I18N_NOOP("KSudoku - Sudokus and more");
 
-static const char version[] = "1.3";
+static const char version[] = "1.4";
 
 
 int main(int argc, char **argv)
 {
-	std::srand(time(0));
+	qsrand(std::time(nullptr));
+	QApplication app(argc, argv);
+	KLocalizedString::setApplicationDomain("ksudoku");
 
-	KAboutData about("ksudoku", 0,
-	                 ki18n("KSudoku"),
+	KAboutData about("ksudoku",
+	                 i18n("KSudoku"),
 	                 version,
-	                 ki18n("KSudoku - Sudoku games and more"),
-	                 KAboutData::License_GPL_V2,
-	                 ki18n("(c) 2005-2007 The KSudoku Authors"),
-	                 KLocalizedString(), "http://ksudoku.sourceforge.net/");
-	about.addAuthor( ki18n("Francesco Rossi"), ki18n("KSudoku Author"), "redsh@email.it" );
-	about.addAuthor( ki18n("Johannes Bergmeier"), ki18n("Maintainer"), "Johannes.Bergmeier@gmx.net" );
-	about.addAuthor( ki18n("Ian Wadham"), ki18n("New puzzle generator and solver"), "iandw.au@gmail.com" );
-	about.addAuthor( ki18n("Mick Kappenburg"), ki18n("Printing and export of 0.4"), "ksudoku@kappendburg.net");
-	about.addAuthor( ki18n("Thanks to NeHe for OpenGL tutorials"), KLocalizedString(), "nehe.gamedev.net");
-	about.addCredit( ki18n("David Bau"), ki18n("Algorithms for new puzzle generator and solver at davidbau.com/archives/2006/09/04/sudoku_generator.html"), "");
-	KCmdLineArgs::init(argc, argv, &about);
+	                 i18n(description),
+	                 KAboutLicense::GPL_V2,
+	                 i18n("(c) 2005-2007 The KSudoku Authors"),
+	                 QString(), "https://games.kde.org/game.php?game=ksudoku");
+	about.addAuthor( i18n("Francesco Rossi"), i18n("KSudoku Author"), "redsh@email.it" );
+	about.addAuthor( i18n("Johannes Bergmeier"), i18n("Maintainer"), "Johannes.Bergmeier@gmx.net" );
+	about.addAuthor( i18n("Ian Wadham"), i18n("New puzzle generator and solver"), "iandw.au@gmail.com" );
+	about.addAuthor( i18n("Mick Kappenburg"), i18n("Printing and export of 0.4"), "ksudoku@kappendburg.net");
+	about.addAuthor( i18n("Thanks to NeHe for OpenGL tutorials"), QString(), "nehe.gamedev.net");
+	about.addCredit( i18n("David Bau"), i18n("Algorithms for new puzzle generator and solver at davidbau.com/archives/2006/09/04/sudoku_generator.html"), "");
 
-	KCmdLineOptions options;
-	options.add("+[URL]", ki18n( "Document to open" ));
-	KCmdLineArgs::addCmdLineOptions(options);
-	KApplication app;
-	KGlobal::locale()->insertCatalog( QLatin1String( "libkdegames" ));
+	KAboutData::setApplicationData(about);
+	app.setOrganizationDomain(QStringLiteral("kde.org"));
+	app.setWindowIcon(QIcon::fromTheme(QStringLiteral("ksudoku")));
+
+	QCommandLineParser parser;
+	about.setupCommandLine(&parser);
+	parser.addVersionOption();
+	parser.addHelpOption();
+	parser.addPositionalArgument(QLatin1String("[URL]"), i18n( "Document to open" ));
+
+	parser.process(app);
+	about.processCommandLine(&parser);
+
+	KCrash::initialize();
 
 	// register ourselves as a dcop client
 //	app.dcopClient()->registerAs(app.name(), false); //TODO PORT
-	
-	 KConfigDialogManager::changedMap()->insert("ksudoku::SymbolConfigListWidget", SIGNAL(itemChanged(QListWidgetItem*)));
+
+	KConfigDialogManager::changedMap()->insert("ksudoku::SymbolConfigListWidget", SIGNAL(itemChanged(QListWidgetItem*)));
 
 	// see if we are starting with session management
 	/*if (app.isRestored())
@@ -81,15 +94,14 @@ int main(int argc, char **argv)
 		widget->show();
 
 		// no session.. just start up normally
-		KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
-		if (args->count() != 0)
+		if (parser.positionalArguments().count() != 0)
 		{
-			for (int i = 0; i < args->count(); ++i)
+			for (int i = 0; i < parser.positionalArguments().count(); ++i)
 			{
-				widget->loadGame(args->url(i));
+				widget->loadGame(QUrl::fromUserInput(parser.positionalArguments().at(i), QDir::currentPath()));
 			}
 		}
-		args->clear();
+
 	//} //TODO PORT
 
 	return app.exec();
